"""
Transformers for all openassessment events
"""

import json

from openedx_caliper_tracking.utils import convert_datetime


def openassessmentblock_get_submission_for_staff_grading(
        current_event, caliper_event):
    """
    When a course team member retrieves a learner's response for grading in the
    staff assessment step the server emits this event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'module': current_event['context']['module'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def openassessmentblock_create_submission(current_event, caliper_event):
    """
    The server emits this event when a learner submits a response.
    The same event is emitted when a learner submits a
    response for peer assessment or for self assessment.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'extensions': {
                'answer': current_event['event']['answer'],
                'attempt_number': current_event['event']['attempt_number'],
                'created_at': convert_datetime(
                    current_event['event']['created_at']),
                'submission_uuid': current_event['event']['submission_uuid'],
                'submitted_at': convert_datetime(
                    current_event['event']['submitted_at'])
            }
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'module': current_event['context']['module']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def openassessmentblock_peer_assess(current_event, caliper_event):
    """
    The server emits this event when a learner submits an assessment of
    peer's response.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = current_event['event']
    event_context_details = current_event['context']

    caliper_object = {
        'dateCreated': convert_datetime(
            current_event_details.pop('scored_at')),
        'extensions': current_event_details,
        'id': current_event['referer'],
        'type': 'Attempt'
    }

    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': event_context_details['asides'],
        'course_user_tags': event_context_details['course_user_tags'],
        'course_id': event_context_details['course_id'],
        'module': event_context_details['module'],
        'ip': current_event['ip']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def openassessment_student_training_assess_example(
        current_event, caliper_event):
    """
    The server emits this event when a learner submits a response.
    The same event is emitted when a learner submits a
    response for peer assessment or for self assessment.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'extensions': {
                'corrections': current_event['event']['corrections'],
                'options_selected': current_event['event']['options_selected'],
                'submission_uuid': current_event['event']['submission_uuid']
            }
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'module': current_event['context']['module']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def openassessmentblock_submit_feedback_on_assessments(
        current_event, caliper_event):
    """
    The server emits openassessmentblock_submit_feedback_on_assessments event
    when learner submit feedback.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.

    """

    module_details = current_event['context']['module']

    caliper_object = {
        'extensions': current_event['event'],
        'id': current_event['referer'],
        'type': 'Message'
    }

    caliper_event.update({
        'action': 'Posted',
        'type': 'MessageEvent',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'display_name': module_details['display_name'],
        'usage_key': module_details['usage_key'],
        'ip': current_event['ip']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def openassessmentblock_save_submission(current_event, caliper_event):
    """
    server emits openassessmentblock.save_submission event when user save
    his response before submitting it.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.

    """
    saved_response = json.loads(current_event['event']['saved_response'])
    module_details = current_event['context']['module']

    caliper_object = {
        'extensions': {
            'saved_response': saved_response
        },
        'id': current_event['referer'],
        'type': 'Assessment'
    }

    caliper_event.update({
        'action': 'Paused',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'display_name': module_details['display_name'],
        'usage_key': module_details['usage_key'],
        'ip': current_event['ip']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def openassessmentblock_get_peer_submission(current_event, caliper_event):
    """
    The server emits openassessmentblock.get_peer_submission event  when
    responses of other course participants are delivered to a learner for
    evaluation.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.

    """

    module_details = current_event['context']['module']

    caliper_object = {
        'extensions': current_event['event'],
        'id': current_event['referer'],
        'type': 'Assessment'
    }

    caliper_event.update({
        'action': 'Viewed',
        'type': 'ViewEvent',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'display_name': module_details['display_name'],
        'usage_key': module_details['usage_key'],
        'ip': current_event['ip']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def openassessmentblock_staff_assess(current_event, caliper_event):
    """
    The server emits this event when a course team member submits an
    assessment of a learner's response.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = current_event['event']
    event_context_details = current_event['context']

    caliper_object = {
        'dateCreated': convert_datetime(current_event_details.pop('scored_at')),
        'extensions': current_event_details,
        'id': current_event['referer'],
        'type': 'Attempt'
    }

    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': event_context_details['asides'],
        'course_user_tags': event_context_details['course_user_tags'],
        'course_id': event_context_details['course_id'],
        'module': event_context_details['module'],
        'ip': current_event['ip']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def openassessmentblock_self_assess(current_event, caliper_event):
    """
    The server emits this event when a learner submits an
    assessment of his own response.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = current_event['event']
    event_context_details = current_event['context']

    caliper_object = {
        'dateCreated': convert_datetime(current_event_details.pop('scored_at')),
        'extensions': current_event_details,
        'id': current_event['referer'],
        'type': 'Attempt'
    }

    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': event_context_details['asides'],
        'course_user_tags': event_context_details['course_user_tags'],
        'course_id': event_context_details['course_id'],
        'module': event_context_details['module'],
        'ip': current_event['ip']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def openassessmentblock_save_files_descriptions(current_event, caliper_event):
    """
    The server emits this event when a description of a learner's response
    file is saved on submission.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'extensions': {
            'saved_response': json.loads(
                current_event['event']['saved_response']
            )
        },
        'id': current_event['referer'],
        'type': 'Document',
        'isPartOf': {
            'id': current_event['referer'],
            'type': 'Assessment'
        }
    }

    caliper_event.update({
        'action': 'Described',
        'type': 'Event',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'ip': current_event['ip'],
        'module': current_event['context']['module'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')
    
    return caliper_event



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='openedx-caliper-tracking',
    version='0.11.1',
    packages=find_packages(),
    include_package_data=True,
    license='GPL 3.0',
    description='Open edX Caliper Tracking app can be used to transform Open edX tracking events into Caliper standard compliant events.',
    long_description=README,
    url='https://github.com/ucsd-ets/caliper-tracking',
    author='UC San Diego',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



"""
Transformers for all the xblock events
"""


def xblock_poll_view_results(current_event, caliper_event):
    """
    The server emits an xblock.poll.view_results event when
    a tally of the responses to a poll is displayed to a user.
    For a poll that has the Private Results option set to False,
    the tally appears after a user submits a response.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Result'
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'Event',
        'action': 'Viewed',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def xblock_survey_view_results(current_event, caliper_event):
    """
    The server emits an xblock.survey.view_results event
    when a matrix of survey response percentages is displayed
    to a user. For surveys that have the Private Results option set
    to False only, the matrix appears after a user submits survey responses.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Result',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'Event',
        'action': 'Viewed',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def xblock_survey_submitted(current_event, caliper_event):
    """
    The server emits an xblock.survey.submitted event each
    time a user submits responses to a survey.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Assessment',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def xblock_poll_submitted(current_event, caliper_event):
    """
    The server emits an xblock.poll.submitted event each
    time a user submits a response to a poll.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Assessment',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def xblock_split_test_child_render(current_event, caliper_event):
    """
    When a student views a module that is set up to test different
    content using child modules, the server emits a xblock.split_test.child_render
    event to identify the child module that was shown to the student.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'DigitalResource',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
    })
    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event



"""
Transformers for all the session related events.
"""
from django.conf import settings

from openedx_caliper_tracking.utils import get_user_link_from_username


def edx_user_login(current_event, caliper_event):
    """
    Server emits this event when user attempts to login.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication',
        'extensions': current_event['event']
    }
    caliper_event.update({
        'action': 'LoggedIn',
        'type': 'SessionEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    username = current_event['event']['username']
    user_link = get_user_link_from_username(username)
    caliper_event['actor'].update({
        'id': user_link,
        'name': username,
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    return caliper_event


def edx_user_logout(current_event, caliper_event):
    """
    This event is emitted when user logged out.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'SessionEvent',
        'action': 'LoggedOut',
        'object': {
            'id': settings.LMS_ROOT_URL,
            'type': 'SoftwareApplication',
            'extensions': current_event['event']
        }
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event.get('ip'),
        'course_id': current_event['context']['course_id']
    })

    return caliper_event



"""
Transformers for all the Third-Party events
"""


def edx_googlecomponent_calendar_displayed(current_event, caliper_event):
    """
    The server emits an edx.googlecomponent.calendar.displayed
    event when a Google Calendar component is shown in the LMS.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Frame',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_googlecomponent_document_displayed(current_event, caliper_event):
    """
    The server emits an edx.googlecomponent.document.displayed
    event when a Google Drive file, such as a document,
    spreadsheet, or image, is shown in the LMS.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def oppia_exploration_state_changed(current_event, caliper_event):
    """
    The server emits an oppia.exploration.state.changed event when
    a user interacts with an Oppia exploration component by submitting
    an answer. Answers are not incorrect or correct.
    All answer submissions change the state of the exploration.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'AssessmentItem',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def oppia_exploration_loaded(current_event, caliper_event):
    """
    The server emits an oppia.exploration.loaded event
    when an Oppia exploration component is shown in the LMS.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'AssessmentItem',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'Event',
        'action': 'Started',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def oppia_exploration_completed(current_event, caliper_event):
    """
    The server emits an oppia.exploration.completed event when
    a user completes an interaction with an Oppia exploration component.
    Oppia explorations do not emit grading events.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'AssessmentItem',
        'extensions': current_event['event']
    }
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'module': current_event['context']['module'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'asides': current_event['context']['asides']
    })
    caliper_event.update({
        'type': 'Event',
        'action': 'Completed',
        'object': caliper_object
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event



"""
Transformers for all cohort related events.
"""

from django.conf import settings
from django.core.urlresolvers import reverse

from openedx_caliper_tracking import utils


def edx_cohort_user_added(current_event, caliper_event):
    """
    When new user is added to cohort.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    username = utils.get_username_from_user_id(
        current_event['event']['user_id'])

    cohort_page_link = '{instructor_page}#view-cohort_management'.format(
        instructor_page=current_event['referer'])

    user_link = '{lms_url}{profile_link}'.format(
        lms_url=settings.LMS_ROOT_URL,
        profile_link=str(reverse(
            'learner_profile',
            kwargs={'username': username}
        ))
    )

    caliper_object = {
        'id': cohort_page_link,
        'member': {
            'extensions': {
                'user_id': current_event['event']['user_id']
            },
            'id': user_link,
            'name': username,
            'type': 'Person'
        },
        'organization': {
            'extensions': {
                'cohort_id': current_event['event']['cohort_id']
            },
            'id': cohort_page_link,
            'name': current_event['event']['cohort_name'],
            'type': 'Group'
        },
        'type': 'Membership'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Added',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    return caliper_event


def edx_cohort_created(current_event, caliper_event):
    """
    Server emits edx.cohort.created event when new cohort is created.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    cohort_page_link = '{instructor_page}#view-cohort_management'.format(
        instructor_page=current_event['referer'])

    caliper_object = {
        'extensions': {
            'cohort_id': current_event['event']['cohort_id']
        },
        'id': cohort_page_link,
        'name': current_event['event']['cohort_name'],
        'type': 'Group'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    return caliper_event


def edx_cohort_user_removed(current_event, caliper_event):
    """
    When user is removed from cohort.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    username = utils.get_username_from_user_id(
        current_event['event']['user_id'])

    user_link = '{lms_url}{profile_link}'.format(
        lms_url=settings.LMS_ROOT_URL,
        profile_link=str(reverse(
            'learner_profile',
            kwargs={'username': username}
        ))
    )

    cohort_page_link = '{instructor_page}#view-cohort_management'.format(
        instructor_page=current_event['referer'])

    caliper_object = {
        'id': cohort_page_link,
        'member': {
            'extensions': {
                'user_id': current_event['event']['user_id']
            },
            'id': user_link,
            'name': username,
            'type': 'Person'
        },
        'organization': {
            'extensions': {
                'cohort_id': current_event['event']['cohort_id']
            },
            'id': cohort_page_link,
            'name': current_event['event']['cohort_name'],
            'type': 'Group'
        },
        'type': 'Membership'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Removed',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    return caliper_event


def edx_cohort_creation_requested(current_event, caliper_event):
    """
    When a course team member manually creates a cohort on the Cohorts page of the instructor
    dashboard, the server emits an edx.cohort.creation_requested event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    cohort_page_link = '{instructor_page}#view-cohort_management'.format(
        instructor_page=current_event['referer'])

    caliper_object = {
        'extensions': {
            'cohort_id': current_event['event']['cohort_id']
        },
        'id': cohort_page_link,
        'name': current_event['event']['cohort_name'],
        'type': 'Group'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
    })

    return caliper_event


def edx_cohort_user_add_requested(current_event, caliper_event):
    """
    When a course team member adds a student to a cohort on the Cohorts page of the instructor
    dashboard, the server emits an edx.cohort.user_add_requested event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    username = utils.get_username_from_user_id(
        current_event['event']['user_id'])

    cohort_page_link = '{instructor_page}#view-cohort_management'.format(
        instructor_page=current_event['referer']
    )

    user_link = utils.get_user_link_from_username(username)

    caliper_object = {
        'id': cohort_page_link,
        'member': {
            'extensions': {
                'user_id': current_event['event']['user_id'],
                'previous_cohort_id': current_event['event']['previous_cohort_id'],
                'previous_cohort_name': current_event['event']['previous_cohort_name'],
            },
            'id': user_link,
            'name': username,
            'type': 'Person'
        },
        'organization': {
            'extensions': {
                'cohort_id': current_event['event']['cohort_id']
            },
            'id': cohort_page_link,
            'name': current_event['event']['cohort_name'],
            'type': 'Group'
        },
        'type': 'Membership'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Added',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    return caliper_event



"""
This module contains the test cases for openedx_caliper_tracking application
"""

import json
import mock
import os

from django.conf import settings
from django.test import TestCase, override_settings

from openedx_caliper_tracking.base_transformer import base_transformer
from openedx_caliper_tracking.caliper_config import EVENT_MAPPING
from openedx_caliper_tracking.processor import CaliperProcessor

TEST_DIR_PATH = os.path.dirname(os.path.abspath(__file__))


class CaliperTransformationTestCase(TestCase):
    """
    Data driven test case that tests all transformers for expected output
    The test case compares the files in the current/ directory
    and compares them with their corresponding files in the expected/ directory
    """

    maxDiff = None

    @mock.patch(
        'openedx_caliper_tracking.utils.get_username_from_user_id',
        return_value='honor',
        autospec=True
    )
    @mock.patch(
        'openedx_caliper_tracking.utils.get_team_url_from_team_id',
        return_value=(
            "http://localhost:18000/courses/course-v1:edX+DemoX+Demo_Course/teams/"
            "#teams/Topic1ID/check-3429fc5983a84c8c8366a4477b03d91c"
        ),
        autospec=True
    )
    def test_caliper_transformers(self, *args):
        """
        Tests whether all the caliper transformers are working as expected
        """
        test_files = [test for test in os.listdir(
            '{}/current/'.format(TEST_DIR_PATH)) if test.endswith(".json")]

        for current_file in test_files:
            input_file = '{}/current/{}'.format(
                TEST_DIR_PATH,
                current_file
            )
            output_file = '{}/expected/{}'.format(
                TEST_DIR_PATH,
                current_file
            )
            with self.settings(LMS_ROOT_URL='http://localhost:18000'):
                with open(input_file) as current, open(output_file) as expected:
                    event = json.loads(current.read())
                    expected_event = json.loads(expected.read())

                    expected_event.pop('id')

                    caliper_event = base_transformer(event)
                    related_function = EVENT_MAPPING[event.get('event_type')]
                    caliper_event = related_function(event, caliper_event)

                    caliper_event.pop('id')

                    self.assertDictEqual(caliper_event, expected_event)


class CaliperDeliveryTestCase(TestCase):
    """
    Test that determines that caliper app is properly delivering the data
    to an external API
    """

    @mock.patch(
        'openedx_caliper_tracking.processor.deliver_caliper_event'
    )
    @override_settings()
    def test_event_is_not_called_without_settings(self, delivery_mock):
        """
        If the settings CALIPER_DELIVER_ENDPOINT and CALIPER_DELIVERY_AUTH_TOKEN are
        not present then the API call shouldn't fire
        """
        del settings.CALIPER_DELIVERY_ENDPOINT
        del settings.CALIPER_DELIVERY_AUTH_TOKEN
        input_file = '{}/current/{}'.format(
            TEST_DIR_PATH,
            'book.json'
        )
        with open(input_file) as current:
            event = json.loads(current.read())

            CaliperProcessor().__call__(event)

        delivery_mock.assert_not_called()

    @mock.patch(
        'openedx_caliper_tracking.processor.deliver_caliper_event'
    )
    @override_settings(
        CALIPER_DELIVERY_ENDPOINT='http://localhost:3000',
        CALIPER_DELIVERY_AUTH_TOKEN='test_auth_token',
    )
    def test_event_is_called_with_settings(self, delivery_mock):
        """
        If the settings are present then the delivery method
        should be called
        """
        input_file = '{}/current/{}'.format(
            TEST_DIR_PATH,
            'book.json'
        )
        with open(input_file) as current:
            event = json.loads(current.read())

            CaliperProcessor().__call__(event)

        delivery_mock.assert_called()



"""
Transformers for all the forums events
"""


def edx_forum_response_created(current_event, caliper_event):
    """
    Users create a reply to a post by clicking Add a Response and then
    submitting their contributions. When these actions are complete,
    the server emits an edx.forum.response.created event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['referer'],
        'body': current_event['event'].pop('body'),
        'extensions': current_event['event'],
        'type': 'Message'
    }

    caliper_event.update({
        'action': 'Posted',
        'type': 'MessageEvent',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })

    return caliper_event


def edx_forum_thread_created(current_event, caliper_event):
    """
    Users create a new top-level thread, also known as a post, by clicking
    New Post and then submitting their contributions. When these actions
    are complete, the server emits an edx.forum.thread.created

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    current_event_details = current_event['event']

    caliper_object = {
        'body': current_event_details.pop('body'),
        'id': current_event['referer'],
        'type': 'Message',
        'name': current_event_details.pop('title'),
        'extensions': current_event_details
    }

    caliper_event.update({
        'type': 'MessageEvent',
        'action': 'Posted',
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_forum_comment_created(current_event, caliper_event):
    """
    Users create a comment about a response by entering text and then
    submitting the contributions. When these actions are complete, the
     server emits an edx.forum.comment.created event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    current_event_details = current_event['event']
    current_event_details.pop('url')

    caliper_object = {
        'body': current_event_details.pop('body'),
        'id': current_event['referer'],
        'type': 'Message',
        'extensions': current_event_details
    }

    caliper_event.update({
        'type': 'MessageEvent',
        'action': 'Posted',
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_forum_thread_viewed(current_event, caliper_event):
    """
    A user views a thread in the course discussions on a desktop, laptop, or
    tablet computer, or on the edX mobile apps.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = current_event['event']
    current_event_details.pop('url', '')

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Thread',
        'name': current_event_details.pop('title', ''),
        'extensions': current_event_details
    }

    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_forum_searched(current_event, caliper_event):
    """
    After a user executes a text search in the navigation sidebar of the
    course Discussion page, the server emits an edx.forum.searched event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Forum',
        'extensions': current_event['event']
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Searched',
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_forum_response_voted(current_event, caliper_event):
    """
    Users can indicate interest in a response by selecting a "Vote" icon.
    The "Vote" icon is a toggle, so users can also clear a vote made
    previously. When either of these actions is complete, the server
    emits an edx.forum.response.voted event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = current_event['event']
    current_event_details.pop('url', '')

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Message',
        'extensions': current_event_details
    }

    caliper_event.update({
        'type': 'Event',
        'action': ('Liked' if current_event_details['vote_value'] == 'up'
                   else 'Disliked'),
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_forum_thread_voted(current_event, caliper_event):
    """
    Users can indicate interest in a thread by selecting a "Vote" icon.
    The "Vote" icon is a toggle, so users can also clear a vote made
    previously. When either of these actions is complete, the server
     emits an edx.forum.thread.voted event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = current_event['event']
    current_event_details.pop('url', '')

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Thread',
        'extensions': current_event_details
    }

    caliper_event.update({
        'type': 'Event',
        'action': ('Liked' if current_event_details['vote_value'] == 'up'
                   else 'Disliked'),
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event
